@@ -129,6 +129,37 @@ setValidity2("DataFrame", .valid.DataFrame)
 ### Constructor.
 ###
 
+### Low-level constructor. For internal use only.
+### Note that, when supplied, 'nrows' is trusted.
+### Calling 'new_DataFrame(x)' on an ordinary named list 'x' will turn it
+### into a DataFrame in the most possibly straightforward way, that is,
+### calling 'as.list()' on the result will return a list identical to 'x'.
+### This is unlike 'DataFrame(x)' or 'as(x, "DataFrame")' which can do all
+### kind of hard-to-predict mangling to 'x', unless the user does something
+### like 'DataFrame(lapply(x, I))'. Not super convenient or intuitive!
+new_DataFrame <- function(listData=list(), nrows=NA, what="arguments")
+{
+        stopifnot(is.list(listData))
+        stopifnot(isSingleNumberOrNA(nrows))
+        if (!is.integer(nrows))
+            nrows <- as.integer(nrows)
+        if (length(listData) == 0L) {
+            if (is.na(nrows))
+                nrows <- 0L
+            names(listData) <- character(0)
+        } else {
+            if (is.na(nrows)) {
+                elt_nrows <- elementNROWS(listData)
+                nrows <- elt_nrows[[1L]]
+                if (!all(elt_nrows == nrows))
+                    stop(wmsg(what, " imply differing number of rows"))
+            }
+            if (is.null(names(listData)))
+                names(listData) <- paste0("V", seq_along(listData))
+        }
+        new2("DataFrame", nrows=nrows, listData=listData, check=FALSE)
+}
+
 DataFrame <- function(..., row.names = NULL, check.names = TRUE)
 {
   ## build up listData, with names from arguments
@@ -137,7 +168,10 @@ DataFrame <- function(..., row.names = NULL, check.names = TRUE)
   nr <- 0
   listData <- list(...)
   varlist <- vector("list", length(listData))
+  metadata <- list()
   if (length(listData) > 0) {
+    if (is(listData[[1L]], getClass("Annotated")))
+        metadata <- metadata(listData[[1L]])
     dotnames <- names(listData)
     if (is.null(dotnames)) {
       emptynames <- rep.int(TRUE, length(listData))
@@ -158,10 +192,12 @@ DataFrame <- function(..., row.names = NULL, check.names = TRUE)
              "\" to a DataFrame")
       nrows[i] <- nrow(element)
       ncols[i] <- ncol(element)
-      varlist[[i]] <- as.list(element, use.names = FALSE)
-      if (!is(listData[[i]], "AsIs")) {
-        if (((length(dim(listData[[i]])) > 1L) || (ncol(element) > 1L) ||
-             is.list(listData[[i]])))
+      varlist[[i]] <- element
+      if (is(listData[[i]], "AsIs")) {
+        listData[[i]] <- drop_AsIs(listData[[i]])
+      } else {
+        if ((length(dim(listData[[i]])) > 1L) || (ncol(element) > 1L) ||
+             is.list(listData[[i]]))
           {
             if (emptynames[i])
               varnames[[i]] <- colnames(element)
@@ -172,6 +208,8 @@ DataFrame <- function(..., row.names = NULL, check.names = TRUE)
       if (missing(row.names))
         row.names <- rownames(element)
     }
+    mcols <- do.call(rbind_mcols, varlist)
+    varlist <- lapply(varlist, as.list, use.names = FALSE)
     nr <- max(nrows)
     for (i in which((nrows > 0L) & (nrows < nr) & (nr %% nrows == 0L))) {
       recycle <- rep(seq_len(nrows[i]), length.out = nr)
@@ -185,7 +223,10 @@ DataFrame <- function(..., row.names = NULL, check.names = TRUE)
     if (check.names)
       nms <- make.names(nms, unique = TRUE)
     names(varlist) <- nms
-  } else names(varlist) <- character(0)
+  } else {
+      names(varlist) <- character(0)
+      mcols <- NULL
+  }
 
   if (!is.null(row.names)) {
     if (anyMissing(row.names))
@@ -197,8 +238,11 @@ DataFrame <- function(..., row.names = NULL, check.names = TRUE)
     row.names <- as.character(row.names)
   }
 
-  new2("DataFrame", listData=varlist, rownames=row.names,
-       nrows=as.integer(max(nr, length(row.names))), check=FALSE)
+  ans <- new_DataFrame(varlist, nrows=as.integer(max(nr, length(row.names))))
+  ans@rownames <- row.names
+  mcols(ans) <- mcols
+  metadata(ans) <- metadata
+  ans
 }
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -277,7 +321,7 @@ setMethod("[", "DataFrame",
                     return(x)
                 j <- i
             }
-            if (!is(j, "Ranges")) {
+            if (!is(j, "IntegerRanges")) {
                 xstub <- setNames(seq_along(x), names(x))
                 j <- normalizeSingleBracketSubscript(j, xstub)
             }
@@ -306,24 +350,76 @@ setMethod("[", "DataFrame",
     }
 )
 
+.make_rownames <- function(x, i, value)
+{
+    x_nrow <- nrow(x)
+    i_max <- max(i)
+    x_rownames <- rownames(x)
+    value_rownames <- rownames(value)
+    if (i_max <= x_nrow || is.null(x_rownames) && is.null(value_rownames))
+        return(x_rownames)
+    ans_rownames <- as.character(seq_len(max(x_nrow, i_max)))
+    if (!is.null(value_rownames))
+        ans_rownames <- replaceROWS(ans_rownames, i, value_rownames)
+    if (is.null(x_rownames))
+        x_rownames <- as.character(seq_len(x_nrow))
+    ans_rownames <- replaceROWS(ans_rownames, seq_len(x_nrow), x_rownames)
+    make.unique(ans_rownames)
+}
+
 setMethod("replaceROWS", "DataFrame",
     function(x, i, value)
     {
-        i <- normalizeSingleBracketSubscript(i, x, as.NSBS=TRUE)
+        i <- normalizeSingleBracketSubscript(i, x, allow.append=TRUE,
+                                             as.NSBS=TRUE)
         x_ncol <- ncol(x)
         value_ncol <- ncol(value)
         if (value_ncol > x_ncol)
             stop("provided ", value_ncol, " variables ",
                  "to replace ", x_ncol, " variables")
-        slot(x, "listData", check=FALSE) <-
-            lapply(structure(seq_len(ncol(x)), names=names(x)),
-                   function(j)
-                       replaceROWS(x[[j]], i,
-                                   value[[((j - 1L) %% value_ncol) + 1L]]))
+        if (x_ncol != 0L) {
+            if (value_ncol == 0L)
+                stop("replacement has length zero")
+            new_listData <-
+                lapply(structure(seq_len(ncol(x)), names=names(x)),
+                       function(j)
+                           replaceROWS(x[[j]], i,
+                                       value[[((j - 1L) %% value_ncol) + 1L]]))
+            slot(x, "listData", check=FALSE) <- new_listData
+        }
+        i <- as.integer(i)
+        i_max <- max(i)
+        x_nrow <- nrow(x)
+        if (i_max > x_nrow) {
+            x@rownames <- .make_rownames(x, i, value)
+            x@nrows <- i_max
+        }
         x
     }
 )
 
+### TODO: Refactor this to use one of the two following 3-step approaches:
+###
+###  (1) Linear `[` + replaceROWS() + linear `[<-`, i.e. something like:
+###        `[<-`(x, j, value=replaceROWS(x[j], i, value))
+###
+###  (2) extractROWS() + linear `[<-` + replaceROWS(), i.e. something like:
+###        replaceROWS(x, i, `[<-`(extractROWS(x, i), j, value=value))
+###
+### (Not sure which one is better, need to figure this out.)
+###
+### Current implementation is a 157-line monolithic function. Doing (1)
+### or (2) means implementing linear `[<-` in a separate helper (e.g.
+### replaceCOLS()) and simplify and reduce the size of the whole thing.
+### Would also make it easier to fix some of the long standing bugs (some
+### of them would just vanish):
+###
+###   `[<-`(DataFrame(aa=1:3), 4, , value=DataFrame(aa=11))  # broken object
+###   DF <- DataFrame(aa=1:3, row.names=LETTERS[1:3])
+###   `[<-`(DF[0], 4, , value=DataFrame(aa=11)[0])  # wrong nrow
+###   `[<-`(DF[0], 2, value=DataFrame(bb=1:3))  # wrong nrow & ncol
+###   etc...
+###
 setReplaceMethod("[", "DataFrame",
                  function(x, i, j, ..., value)
                  {
@@ -369,10 +465,14 @@ setReplaceMethod("[", "DataFrame",
                    j <- j2
                    if (!length(j)) # nothing to replace
                      return(x)
-                   if (is(value, "list") || is(value, "List")) {
-                     null <- vapply(value, is.null, logical(1L))
-                     if (any(null)) { ### FIXME: data.frame handles gracefully
-                       stop("NULL elements not allowed in list value")
+                   if (is(value, "list_OR_List") &&
+                       pcompareRecursively(value))
+                   {
+                     if (is(value, "List") && is(NULL, elementType(value))) {
+                       null <- sapply_isNULL(value)
+                       if (any(null)) { ### FIXME: data.frame handles gracefully
+                         stop("NULL elements not allowed in list value")
+                       }
                      }
                      value <- as(value, "DataFrame")
                    }
@@ -467,8 +567,9 @@ setReplaceMethod("[", "DataFrame",
                      oldcn <- head(colnames(x), length(x) - length(newcn))
                      colnames(x) <- make.unique(c(oldcn, newcn))
                      if (!is.null(mcols(x)))
-                       mcols(x)[tail(names(x),length(newcn)),] <-
-                         DataFrame(NA)
+                       mcols(x) <- replaceROWS(mcols(x),
+                                               tail(names(x), length(newcn)),
+                                               DataFrame(NA))
                    }
                    if (length(newrn)) {
                      notj <- setdiff(seq_len(ncol(x)), j)
@@ -477,7 +578,7 @@ setReplaceMethod("[", "DataFrame",
                               function(y) c(y, rep(NA, length(newrn))))
                      x@rownames <- make.unique(c(rownames(x), newrn))
                    }
-                   x@nrows <- length(x[[1]]) # we should always have a column
+                   x@nrows <- NROW(x[[1]]) # we should always have a column
                    x
                  })
 
@@ -566,7 +667,9 @@ setAs("data.frame", "DataFrame",
 ### But unclass() causes deep copy
         attr(from, "row.names") <- NULL
         class(from) <- NULL
-        new2("DataFrame", listData=from, nrows=nr, rownames=rn, check=FALSE)
+        ans <- new_DataFrame(from, nrows=nr)
+        ans@rownames <- rn
+        ans
       })
 
 setAs("data.table", "DataFrame",
@@ -601,8 +704,9 @@ setAs("xtabs", "DataFrame",
     as(df, "DataFrame")
   } else {
     row.names <- if (!anyDuplicated(names(from))) names(from) else NULL
-    new2("DataFrame", listData = setNames(list(from), "X"),
-         nrows = length(from), rownames = row.names, check=FALSE)
+    ans <- new_DataFrame(setNames(list(from), "X"), nrows=length(from))
+    ans@rownames <- row.names
+    ans
   }
 }
 
@@ -638,24 +742,28 @@ setAs("integer", "DataFrame",
 
 setAs("AsIs", "DataFrame",
       function(from) {
-        df <- new2("DataFrame", nrows = NROW(from), check=FALSE)
-        df[[1]] <- from
-        df
+        new_DataFrame(setNames(list(drop_AsIs(from)), "X"))
       })
 
-setAs("ANY", "AsIs", function(from) I(from))
-
 setAs("ANY", "DataTable_OR_NULL", function(from) as(from, "DataFrame"))
 
+### Turn an ordinary list into a DataFrame in the most possibly
+### straightforward way.
+setMethod("coerce2", "DataFrame",
+    function(from, to)
+    {
+        if (class(from) != "list")
+            return(callNextMethod())
+        new_DataFrame(from, what="list elements")
+    }
+)
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Combining.
 ###
 
 cbind.DataFrame <- function(..., deparse.level = 1) {
-  dfs <- lapply(list(...), as, "DataFrame", strict=FALSE)
-  ans <- DataFrame(dfs)
-  mcols(ans) <- do.call(rbind_mcols, dfs)
-  ans
+  DataFrame(..., check.names=FALSE)
 }
 
 setMethod("cbind", "DataFrame", cbind.DataFrame)
@@ -665,22 +773,33 @@ rbind.DataFrame <- function(..., deparse.level = 1) {
 }
 
 setMethod("rbind", "DataFrame", function(..., deparse.level=1) {
-  args <- list(...)
-  hasrows <- unlist(lapply(args, nrow), use.names=FALSE) > 0L
-  hascols <- unlist(lapply(args, ncol), use.names=FALSE) > 0L
+  args <- list(...)    
+  
+  hasrows <- elementNROWS(args) > 0L
+  hascols <- vapply(args, ncol, integer(1L)) > 0L
 
+  ans <- NULL
   if (!any(hasrows | hascols)) {
-    return(DataFrame())
+    ans <- DataFrame()
   } else if (!any(hasrows)) {
-    return(args[[which(hascols)[1L]]])
+    ans <- args[[which(hascols)[1L]]]
   } else if (sum(hasrows) == 1) {
-    return(args[[which(hasrows)]])
+    ans <- args[[which(hasrows)]]
   } else {
     args <- args[hasrows]
   }
 
   df <- args[[1L]]
 
+  ans_metadata <- metadata(df)
+  ans_mcols <- mcols(df)
+  
+  if (!is.null(ans)) {
+      metadata(ans) <- ans_metadata
+      mcols(ans) <- ans_mcols
+      return(ans)
+  }
+  
   for (i in 2:length(args)) {
     if (ncol(df) != ncol(args[[i]]))
       stop("number of columns for arg ", i, " do not match those of first arg")
@@ -704,9 +823,10 @@ setMethod("rbind", "DataFrame", function(..., deparse.level=1) {
         levs <- unique(unlist(lapply(cols, levels), use.names=FALSE))
         cols <- lapply(cols, factor, levs)
       }
-      rectangular <- length(dim(cols[[1]])) == 2L
-      if (rectangular) {
+      matrix_like <- length(dim(cols[[1]])) == 2L
+      if (matrix_like) {
         combined <- do.call(rbind, unname(cols))
+        ### FIXME: handle the general array case (arbind)
       } else {
         combined <- do.call(c, unname(cols))
       }
@@ -715,8 +835,7 @@ setMethod("rbind", "DataFrame", function(..., deparse.level=1) {
       combined
     })
     names(cols) <- colnames(df)
-    ans <- new2("DataFrame", listData = cols, nrows = NROW(cols[[1]]),
-                             check = FALSE)
+    ans <- new_DataFrame(cols, nrows=NROW(cols[[1]]))
   }
 
   rn <- unlist(lapply(args, rownames), use.names=FALSE)
@@ -727,13 +846,10 @@ setMethod("rbind", "DataFrame", function(..., deparse.level=1) {
       rn <- make.unique(rn, sep = "")
   }
   rownames(ans) <- rn
-
-  if (!is.null(mcols(df))) {
-    df_mcols <- mcols(df)
-    if (all(sapply(args, function(x) identical(mcols(x), df_mcols))))
-      mcols(ans) <- df_mcols
-  }
-
+  
+  metadata(ans) <- ans_metadata
+  mcols(ans) <- ans_mcols
+ 
   ans
 })
 
