@@ -15,6 +15,8 @@ setClass("List",
     prototype(elementType="ANY")
 )
 
+setClassUnion("list_OR_List", c("list", "List"))  # list-like objects
+
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Accessor methods.
@@ -57,7 +59,7 @@ setMethod("isEmpty", "ANY",
           {
               if (is.atomic(x))
                   return(length(x) == 0L)
-              if (!is.list(x) && !is(x, "List"))
+              if (!is(x, "list_OR_List"))
                   stop("isEmpty() is not defined for objects of class ",
                        class(x))
               ## Recursive definition
@@ -263,9 +265,9 @@ setMethod("unlist", "List",
             return("NL")
         }
     }
-    if (extends(i_LEPType, "Ranges")) {
-        ## 'i' is a List of Ranges objects.
-        ## We select the "RL" fast path ("Ranges List").
+    if (extends(i_LEPType, "IntegerRanges")) {
+        ## 'i' is a List of IntegerRanges objects.
+        ## We select the "RL" fast path ("IntegerRanges List").
         return("RL")
     }
     return(NA_character_)
@@ -305,7 +307,7 @@ setMethod("unlist", "List",
 }
 
 ### Assume 'x' and 'i' are parallel List objects (i.e. same length),
-### and 'i' is a List of Ranges objects.
+### and 'i' is a List of IntegerRanges objects.
 .unlist_RL_subscript <- function(i, x)
 {
     unlisted_i <- unlist(i, use.names=FALSE)
@@ -329,9 +331,9 @@ setMethod("unlist", "List",
     ## Relist.
     group <- rep.int(seq_along(x), elementNROWS(x))
     group <- extractROWS(group, unlisted_i)
-    ans_skeleton <- IRanges::PartitioningByEnd(group, NG=length(x),
-                                               names=names(x))
-    relist(unlisted_ans, ans_skeleton)
+    ans_partitioning <- IRanges::PartitioningByEnd(group, NG=length(x),
+                                                   names=names(x))
+    relist(unlisted_ans, ans_partitioning)
 }
 
 ### Fast subset by List of numeric vectors or numeric-Rle objects.
@@ -349,16 +351,23 @@ setMethod("unlist", "List",
 
     ## Relist.
     ans_breakpoints <- cumsum(unname(elementNROWS(i)))
-    ans_skeleton <- IRanges::PartitioningByEnd(ans_breakpoints, names=names(x))
-    relist(unlisted_ans, ans_skeleton)
+    ans_partitioning <- IRanges::PartitioningByEnd(ans_breakpoints,
+                                                   names=names(x))
+    relist(unlisted_ans, ans_partitioning)
 }
 
-### Fast subset by List of Ranges objects.
+### Fast subset by List of IntegerRanges objects.
 ### Assume 'x' and 'i' are parallel List objects (i.e. same length).
 ### Propagate 'names(x)' only. Caller is responsible for propagating 'mcols(x)'
 ### and 'metadata(x)'.
 .fast_subset_List_by_RL <- function(x, i)
 {
+    i_eltNROWS <- elementNROWS(i)
+    if (all(i_eltNROWS == 1L)) {
+        unlisted_i <- unlist(i, use.names=FALSE)
+        return(IRanges::windows(x, unlisted_i))
+    }
+
     ## Unlist 'x' and 'i'.
     unlisted_x <- unlist(x, use.names=FALSE)
     unlisted_i <- .unlist_RL_subscript(i, x)
@@ -368,8 +377,9 @@ setMethod("unlist", "List",
 
     ## Relist.
     ans_breakpoints <- cumsum(unlist(sum(width(i)), use.names=FALSE))
-    ans_skeleton <- IRanges::PartitioningByEnd(ans_breakpoints, names=names(x))
-    relist(unlisted_ans, ans_skeleton)
+    ans_partitioning <- IRanges::PartitioningByEnd(ans_breakpoints,
+                                                   names=names(x))
+    relist(unlisted_ans, ans_partitioning)
 }
 
 ### Subset a List object by a list-like subscript.
@@ -525,13 +535,16 @@ lsubset_List_by_List <- function(x, i, value)
 setMethod("[", "List",
     function(x, i, j, ..., drop=TRUE)
     {
-        if (!missing(j) || length(list(...)) > 0L)
+        if (length(list(...)) > 0L)
             stop("invalid subsetting")
-        if (missing(i))
-            return(x)
-        if (is.list(i) || (is(i, "List") && !is(i, "Ranges")))
-            return(subset_List_by_List(x, i))
-        callNextMethod(x, i)
+        if (missing(i) || !is(i, "list_OR_List") || is(i, "IntegerRanges")) {
+            ans <- subset_along_ROWS(x, i, drop=drop)
+        } else {
+            ans <- subset_List_by_List(x, i)
+        }
+        if (!missing(j))
+            mcols(ans) <- mcols(ans)[ , j, drop=FALSE]
+        ans
     }
 )
 
@@ -540,8 +553,8 @@ setReplaceMethod("[", "List",
     {
         if (!missing(j) || length(list(...)) > 0L)
             stop("invalid subsetting")
-        if (!missing(i) && (is.list(i) || (is(i, "List") && !is(i, "Ranges"))))
-                return(lsubset_List_by_List(x, i, value))
+        if (!missing(i) && is(i, "list_OR_List") && !is(i, "IntegerRanges"))
+            return(lsubset_List_by_List(x, i, value))
         callNextMethod(x, i, value=value)
     }
 )
@@ -562,16 +575,13 @@ setMethod("[[", "List",
 setMethod("$", "List", function(x, name) x[[name, exact=FALSE]])
 
 setReplaceMethod("[[", "List",
-                 function(x, i, j, ..., value)
-                 {
-                   if (!missing(j) || length(list(...)) > 0)
-                     stop("invalid replacement")
-                   origLen <- length(x)
-                   x <- setListElement(x, i, value)
-                   if (origLen < length(x))
-                     x <- rbindRowOfNAsToMetadatacols(x)
-                   x
-                 })
+    function(x, i, j, ..., value)
+    {
+        if (!missing(j) || length(list(...)) > 0)
+            stop("invalid replacement")
+        setListElement(x, i, value)
+    }
+)
 
 setReplaceMethod("$", "List",
                  function(x, name, value) {
@@ -579,50 +589,23 @@ setReplaceMethod("$", "List",
                    x
                  })
 
+setMethod("setListElement", "List", setListElement_default)
+
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Simple helper functions for some common subsetting operations.
 ###
-### TODO: Move to List-utils.R (Looping methods section).
-###
-### phead() and ptail(): "parallel" versions of head() and tail() for List
-### objects. They're just fast equivalents of 'mapply(head, x, n)' and
-### 'mapply(tail, x, n)', respectively.
-.normarg_n <- function(n, x_eltNROWS)
-{
-    if (!is.numeric(n))
-        stop("'n' must be an integer vector")
-    if (!is.integer(n))
-        n <- as.integer(n)
-    if (any(is.na(n)))
-        stop("'n' cannot contain NAs")
-    n <- pmin(x_eltNROWS, n)
-    neg_idx <- which(n < 0L)
-    if (length(neg_idx) != 0L)
-        n[neg_idx] <- pmax(n[neg_idx] + x_eltNROWS[neg_idx], 0L)
-    n
-}
 
 phead <- function(x, n=6L)
 {
-    x_eltNROWS <- unname(elementNROWS(x))
-    n <- .normarg_n(n, x_eltNROWS)
-    unlisted_i <- IRanges::IRanges(start=rep.int(1L, length(n)), width=n)
-    i <- relist(unlisted_i, IRanges::PartitioningByEnd(seq_along(x)))
-    ans <- x[i]
-    mcols(ans) <- mcols(x)
-    ans
+    .Deprecated("IRanges::heads")
+    IRanges::heads(x, n=n)
 }
 
 ptail <- function(x, n=6L)
 {
-    x_eltNROWS <- unname(elementNROWS(x))
-    n <- .normarg_n(n, x_eltNROWS)
-    unlisted_i <- IRanges::IRanges(end=x_eltNROWS, width=n)
-    i <- relist(unlisted_i, IRanges::PartitioningByEnd(seq_along(x)))
-    ans <- x[i]
-    mcols(ans) <- mcols(x)
-    ans
+    .Deprecated("IRanges::tails")
+    IRanges::tails(x, n=n)
 }
 
 
