@@ -9,40 +9,36 @@
 ###
 
 setMethod("lapply", "List",
-          function(X, FUN, ...)
-          {
-              FUN <- match.fun(FUN)
-              ii <- seq_len(length(X))
-              names(ii) <- names(X)
-              lapply(ii, function(i) FUN(X[[i]], ...))
-          })
+    function(X, FUN, ...)
+    {
+        FUN <- match.fun(FUN)
+        ii <- setNames(seq_along(X), names(X))
+        lapply(ii, function(i) FUN(X[[i]], ...))
+    }
+)
 
 .sapplyDefault <- base::sapply
 environment(.sapplyDefault) <- topenv()
 setMethod("sapply", "List", .sapplyDefault)
 
-setGeneric("endoapply", signature = "X",
-           function(X, FUN, ...) standardGeneric("endoapply"))
+### Turn ordinary list 'ans' into an object of the same class as list-like
+### object 'X'. Preserve the length and names of 'ans'. Propagate the metadata
+### and metadata columns from 'X'.
+.make_endoapply_ans <- function(ans, X)
+{
+    ans <- coerce2(ans, X)
+    if (is(X, "Vector")) {
+        metadata(ans) <- metadata(X)
+        mcols(ans) <- mcols(X)
+    }
+    ans
+}
 
-setMethod("endoapply", "list",
-          function(X, FUN, ...) lapply(X = X, FUN = match.fun(FUN), ...))
-
-setMethod("endoapply", "data.frame",
-          function(X, FUN, ...)
-          as.data.frame(lapply(X = X, FUN = match.fun(FUN), ...)))
-
-setMethod("endoapply", "List",
-          function(X, FUN, ...) {
-              elementTypeX <- elementType(X)
-              FUN <- match.fun(FUN)
-              for (i in seq_len(length(X))) {
-                  elt <- FUN(X[[i]], ...)
-                  if (!extends(class(elt), elementTypeX))
-                      stop("'FUN' must return elements of class ", elementTypeX)
-                  X[[i]] <- elt
-              }
-              X
-          })
+endoapply <- function(X, FUN, ...)
+{
+    ans <- lapply(X, FUN, ...)
+    .make_endoapply_ans(ans, X)
+}
 
 setGeneric("revElements", signature="x",
     function(x, i) standardGeneric("revElements")
@@ -67,31 +63,12 @@ setMethod("revElements", "List",
     }
 )
 
-setGeneric("mendoapply", signature = "...",
-           function(FUN, ..., MoreArgs = NULL) standardGeneric("mendoapply"))
-BiocGenerics:::apply_hotfix73465(getGeneric("mendoapply"))
-
-setMethod("mendoapply", "list", function(FUN, ..., MoreArgs = NULL)
-          mapply(FUN = FUN, ..., MoreArgs = MoreArgs, SIMPLIFY = FALSE))
-
-setMethod("mendoapply", "data.frame", function(FUN, ..., MoreArgs = NULL)
-          as.data.frame(mapply(FUN = match.fun(FUN), ..., MoreArgs = MoreArgs,
-                               SIMPLIFY = FALSE)))
-
-setMethod("mendoapply", "List",
-          function(FUN, ..., MoreArgs = NULL) {
-              X <- list(...)[[1L]]
-              elementTypeX <- elementType(X)
-              FUN <- match.fun(FUN)
-              listData <-
-                mapply(FUN = FUN, ..., MoreArgs = MoreArgs, SIMPLIFY = FALSE)
-              for (i in seq_len(length(listData))) {
-                  if (!extends(class(listData[[i]]), elementTypeX))
-                      stop("'FUN' must return elements of class ", elementTypeX)
-                  X[[i]] <- listData[[i]]
-              }
-              X
-          })
+mendoapply <- function(FUN, ..., MoreArgs=NULL)
+{
+    arg1 <- list(...)[[1L]]
+    ans <- mapply(FUN, ..., MoreArgs=MoreArgs, SIMPLIFY=FALSE)
+    .make_endoapply_ans(ans, arg1)
+}
 
 ### Element-wise c() for list-like objects.
 ### This is a fast mapply(c, ..., SIMPLIFY=FALSE) but with the following
@@ -219,7 +196,7 @@ setMethod("within", "List",
             e <- list2env(as.list(data), parent=enclos)
             safeEval(substitute(expr), e, enclos)
             l <- mget(ls(e), e)
-            l <- l[!sapply(l, is.null)]
+            l <- delete_NULLs(l)
             nD <- length(del <- setdiff(names(data), (nl <- names(l))))
             for (nm in nl)
               data[[nm]] <- l[[nm]]
