@@ -14,16 +14,8 @@ setClass("Pairs",
                    second=logical(0L),
                    elementMetadata=DataFrame()))
 
-setValidity2("Pairs", .valid.Pairs)
-
-.valid.Pairs <- function(object) {
-    c(if (length(object@first) != length(object@second))
-          "'first' and 'second' must have the same length",
-      if (!is.null(object@NAMES) &&
-          length(object@NAMES) != length(object@first))
-          "'NAMES', if not NULL, must have the same length as object"
-      )
-}
+setMethod("parallelSlotNames", "Pairs", function(x)
+    c("first", "second", "NAMES", callNextMethod()))
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Accessors
@@ -55,11 +47,6 @@ setReplaceMethod("names", "Pairs", function(x, value) {
                      x
                  })
 
-setMethod("length", "Pairs", function(x) length(first(x)))
-
-setMethod("parallelSlotNames", "Pairs", function(x)
-    c("first", "second", "NAMES", callNextMethod()))
-
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Constructor
 ###
@@ -153,28 +140,6 @@ setMethod("as.data.frame", "Pairs",
                             row.names=row.names, ...)
           })
 
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Combine
-###
-
-.unlist_list_of_Pairs <- function(x) {
-    Pairs(do.call(c, lapply(x, first)),
-          do.call(c, lapply(x, second)),
-          do.call(rbind, lapply(x, mcols)),
-### FIXME: breaks if only some names are NULL
-          names = unlist(lapply(x, names)))
-}
-
-setMethod("c", "Pairs", function (x, ..., recursive = FALSE) {
-    if (!identical(recursive, FALSE)) 
-        stop("'recursive' argument not supported")
-    if (missing(x))
-        args <- unname(list(...))
-    else args <- unname(list(x, ...))
-    .unlist_list_of_Pairs(args)
-})
-
-
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Utilities
 ###
