@@ -67,7 +67,8 @@ setValidity2("Rle", .valid_Rle)
 ### Low-level constructor.
 new_Rle <- function(values=logical(0), lengths=NULL)
 {
-    stopifnot(is(values, "vector_OR_factor"))
+    if (!is(values, "vector_OR_factor"))
+        stop("Rle of type '", typeof(values), "' is not supported")
     if (!is.null(lengths)) {
         if (!(is.numeric(lengths) || is.LLint(lengths)))
             stop("'lengths' must be NULL or a numeric or LLint vector")
@@ -122,14 +123,7 @@ setReplaceMethod("runValue", "Rle",
 ### Coercion
 ###
 
-setAs("vector", "Rle", function(from) Rle(from))
-setAs("logical", "Rle", function(from) Rle(from))
-setAs("integer", "Rle", function(from) Rle(from))
-setAs("numeric", "Rle", function(from) Rle(from))
-setAs("complex", "Rle", function(from) Rle(from))
-setAs("character", "Rle", function(from) Rle(from))
-setAs("raw", "Rle", function(from) Rle(from))
-setAs("factor", "Rle", function(from) Rle(from))
+setAs("ANY", "Rle", function(from) Rle(from))
 
 setAs("Rle", "vector", function(from) as.vector(from))
 setAs("Rle", "logical", function(from) as.logical(from))
@@ -300,7 +294,7 @@ setMethod("replaceROWS", "Rle",
 
         ## From here, 'value' is guaranteed to be of length 1.
 
-        ## TODO: Maybe make this the coercion method from NSBS to Ranges.
+        ## TODO: Maybe make this the coercion method from NSBS to IntegerRanges.
         if (is(i, "RangesNSBS")) {
             ir <- i@subscript
         } else {
@@ -446,8 +440,8 @@ setMethod("NSBS", "Rle",
             ## IRanges package.
             if (requireNamespace("IRanges", quietly=TRUE)) {
                 i <- as(i, "NormalIRanges")
-                ## This will call the "NSBS" method for Ranges objects defined
-                ## in the IRanges package and return a RangesNSBS, or
+                ## This will call the "NSBS" method for IntegerRanges objects
+                ## defined in the IRanges package and return a RangesNSBS, or
                 ## RangeNSBS, or NativeNSBS object.
                 return(callGeneric())
             }
@@ -541,7 +535,7 @@ setMethod("rep.int", "Rle",
             stop("invalid 'times' argument")
         ans <- Rle(rep.int(runValue(x), times),
                    rep.int(runLength(x), times))
-        as(ans, class(x))  # rep.int() must act like an endomorphism
+        as(ans, class(x))  # so the function is an endomorphism
     }
 )
 
@@ -592,24 +586,43 @@ setMethod("rep", "Rle",
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Combining.
+### Concatenation
 ###
 
-setMethod("c", "Rle", 
-          function(x, ..., recursive = FALSE)
-          {
-              args <- lapply(unname(list(x, ...)), Rle)
-              args_values <- lapply(args, slot, "values")
-              ## use unlist() for factors:
-              ans_values <- unlist(args_values, recursive=FALSE)
-              if (is.null(ans_values)) {
-                  ## use c() to get type promotion right in zero-length case:
-                  ans_values <- do.call(c, args_values)
-              }
-              ans_lengths <- unlist(lapply(args, slot, "lengths"),
-                                    recursive=FALSE)
-              Rle(ans_values, ans_lengths)
-          })
+.concatenate_Rle_objects <-
+    function(x, objects=list(), use.names=TRUE, ignore.mcols=FALSE, check=TRUE)
+{
+    objects <- prepare_objects_to_concatenate(x, objects)
+    all_objects <- c(list(x), objects)
+
+    ## 1. Take care of the parallel slots
+
+    ## Call method for Vector objects to concatenate all the parallel
+    ## slots (only "elementMetadata" in the case of Rle) and stick them
+    ## into 'ans'. Note that the resulting 'ans' can be an invalid object
+    ## because its "elementMetadata" slot can be longer (i.e. have more rows)
+    ## than 'ans' itself so we use 'check=FALSE' to skip validation.
+    ans <- callNextMethod(x, objects, use.names=use.names,
+                                      ignore.mcols=ignore.mcols,
+                                      check=FALSE)
+
+    ## 2. Take care of the non-parallel slots
+
+    ## Concatenate the "values" slots.
+    values_list <- lapply(all_objects, slot, "values")
+    tmp_values <- unlist(values_list, recursive=FALSE)
+
+    ## Concatenate the "lengths" slots.
+    lengths_list <- lapply(all_objects, slot, "lengths")
+    tmp_lengths <- unlist(lengths_list, recursive=FALSE)
+
+    tmp <- Rle(tmp_values, tmp_lengths)
+    BiocGenerics:::replaceSlots(ans, values=tmp@values,
+                                     lengths=tmp@lengths,
+                                     check=check)
+}
+
+setMethod("concatenateObjects", "Rle", .concatenate_Rle_objects)
 
 setMethod("append", c("Rle", "vector"),
           function (x, values, after = length(x)) {
