@@ -4,6 +4,22 @@
 ###
 
 
+### Not really a S4-related utility but I don't have a better place to put
+### this at the moment.
+drop_AsIs <- function(x)
+{
+    x_class <- class(x)
+    class(x) <- x_class[x_class != "AsIs"]
+    x
+}
+
+setAs("ANY", "AsIs", function(from) I(from))
+
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Some convenient union classes
+###
+
 setClassUnion("character_OR_NULL", c("character", "NULL"))
 
 ### WARNING: The behavior of is.vector(), is( , "vector"), is.list(), and
@@ -66,6 +82,11 @@ setClassUnion("vector_OR_factor", c("vector", "factor"))
 ###   the "mode" argument makes things complicated.
 setAs("ANY", "vector", function(from) as.vector(from))
 
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Coercion utilities
+###
+
 coercerToClass <- function(class) {
   if (extends(class, "vector"))
     .as <- get(paste0("as.", class))
@@ -79,6 +100,47 @@ coercerToClass <- function(class) {
   }
 }
 
+### A version of coerce() that tries to do a better job at coercing to an
+### S3 class. Dispatch on the 2nd argument only!
+setGeneric("coerce2", signature="to",
+    function(from, to) standardGeneric("coerce2")
+)
+
+### TODO: Should probably use coercerToClass() internally (but coercerToClass()
+### would first need to be improved).
+setMethod("coerce2", "ANY",
+    function(from, to)
+    {
+        to_class <- class(to)
+        if (is(from, to_class))
+            return(from)
+        if (is.data.frame(to)) {
+            ans <- as.data.frame(from, check.names=FALSE,
+                                       stringsAsFactors=FALSE)
+        } else {
+            S3coerceFUN <- try(match.fun(paste0("as.", to_class)),
+                               silent=TRUE)
+            if (!inherits(S3coerceFUN, "try-error")) {
+                ans <- S3coerceFUN(from)
+            } else {
+                ans <- as(from, to_class, strict=FALSE)
+            }
+        }
+        if (length(ans) != length(from))
+            stop(wmsg("coercion of ", class(from), " object to ", to_class,
+                      " didn't preserve its length"))
+        ## Try to restore the names if they were lost (e.g. by as.integer())
+        ## or altered (e.g. by as.data.frame(), which will alter names equal
+        ## to the empty string even if called with 'check.names=FALSE').
+        if (!identical(names(ans), names(from))) {
+            tmp <- try(`names<-`(ans, value=names(from)) , silent=TRUE)
+            if (!inherits(tmp, "try-error"))
+                ans <- tmp
+        }
+        ans
+    }
+)
+
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### setValidity2(), new2()
