@@ -2,18 +2,29 @@
 ### SimpleList objects
 ### -------------------------------------------------------------------------
 
+
 setClass("SimpleList",
-         contains="List",
-         representation(
-                        listData="list"
-                        )
-         )
+    contains="List",
+    representation(
+        listData="list"
+    )
+)
+
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Accessor methods.
+### parallelSlotNames()
 ###
 
-setMethod("length", "SimpleList", function(x) length(as.list(x)))
+### Combine the new parallel slots with those of the parent class. Make sure
+### to put the new parallel slots *first*.
+setMethod("parallelSlotNames", "SimpleList",
+    function(x) c("listData", callNextMethod())
+)
+
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Accessor methods
+###
 
 setMethod("names", "SimpleList", function(x) names(as.list(x)))
 
@@ -23,8 +34,9 @@ setReplaceMethod("names", "SimpleList",
                      x
                  })
 
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Constructor.
+### Constructor
 ###
 
 ### Low-level. NOT exported.
@@ -45,7 +57,11 @@ new_SimpleList_from_list <- function(Class, x, ..., mcols)
         names(x) <- tmp_names
     }
     class(x) <- "list"
-    ans_elementType <- elementType(new(Class))
+    proto <- new(Class)
+    ans_elementType <- elementType(proto)
+    if (is(S4Vectors::mcols(proto), "DataFrame")) {
+        mcols <- make_zero_col_DataFrame(length(x))
+    }
     if (!all(sapply(x, function(xi) extends(class(xi), ans_elementType))))
         stop("all elements in 'x' must be ", ans_elementType, " objects")
     if (missing(mcols))
@@ -64,7 +80,7 @@ SimpleList <- function(...)
     ## create later with new(). For example if we were using is.list() instead
     ## of extends(), the test would pass on matrix(list()) but new() then would
     ## fail with the following message:
-    ## Error in validObject(.Object) : 
+    ## Error in validObject(.Object) :
     ##   invalid class “SimpleList” object: invalid object for slot "listData"
     ##   in class "SimpleList": got class "matrix", should be or extend class
     ##   "list"
@@ -75,7 +91,7 @@ SimpleList <- function(...)
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Validity.
+### Validity
 ###
 
 .valid.SimpleList.listData <- function(x)
@@ -104,82 +120,79 @@ setMethod("classNameForDisplay", "SimpleList",
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Subsetting.
+### Subsetting
 ###
 
-setMethod("parallelSlotNames", "SimpleList",
-          function(x) c("listData", callNextMethod()))
-
 setMethod("getListElement", "SimpleList",
     function(x, i, exact=TRUE)
         getListElement(x@listData, i, exact=exact)
 )
 
-setMethod("setListElement", "SimpleList",
-    function(x, i, value)
-    {
-        x@listData[[i]] <- value
-        x
-    }
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Looping
+###
+
+setMethod("lapply", "SimpleList",
+    function(X, FUN, ...) lapply(as.list(X), match.fun(FUN), ...)
 )
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Combining.
+### Coercion
 ###
 
-## NOTE: while the 'c' function does not have an 'x', the generic does
-## c() is a primitive, so 'x' can be missing; dispatch is by position,
-## although sometimes this does not work so well, so it's best to keep
-## names off the parameters whenever feasible.
-
-#setMethod("c", "SimpleList",
-#          function(x, ..., recursive = FALSE) {
-#              slot(x, "listData") <-
-#                do.call(c, lapply(unname(list(x, ...)), as.list))
-#              if (!is.null(mcols(x)))
-#                mcols(x) <- rbind.mcols(x, ...)
-#              x
-#          })
-
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Looping.
+### Unfortunately, not all SimpleList subclasses (e.g. BamFileList or
+### ExperimentList) support coercion from an ordinary list (even though
+### they probably should), so the default "coerce2" method will fail to
+### convert an ordinary list to one of these classes. The good news is that
+### coercion from SimpleList to one of these classes does work. For example:
 ###
-
-### TODO: easily generalized to List
-setMethod("lapply", "SimpleList",
-          function(X, FUN, ...)
-              lapply(as.list(X), FUN = FUN, ...))
-
-setMethod("endoapply", "SimpleList",
-          function(X, FUN, ...) {
-              FUN <- match.fun(FUN)
-              listData <- lapply(X, FUN = FUN, ...)
-              elementTypeX <- elementType(X)
-              if (!all(sapply(listData,
-                              function(Xi) extends(class(Xi), elementTypeX))))
-                  stop("all results must be of class '", elementTypeX, "'")
-              slot(X, "listData", check=FALSE) <- listData
-              X
-          })
-
-setMethod("mendoapply", "SimpleList",
-          function(FUN, ..., MoreArgs = NULL) {
-              X <- list(...)[[1L]]
-              elementTypeX <- elementType(X)
-              FUN <- match.fun(FUN)
-              listData <-
-                mapply(FUN = FUN, ..., MoreArgs = MoreArgs, SIMPLIFY = FALSE)
-              if (!all(sapply(listData,
-                              function(Xi) extends(class(Xi), elementTypeX))))
-                  stop("all results must be of class '", elementTypeX, "'")
-              slot(X, "listData", check=FALSE) <- listData
-              X
-          })
-
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Coercion.
+###   library(Rsamtools)
+###   as(list(), "BamFileList")           # error
+###   as(SimpleList(), "BamFileList")     # works
 ###
+###   library(MultiAssayExperiment)
+###   as(list(), "ExperimentList")        # error
+###   as(SimpleList(), "ExperimentList")  # works
+###
+### So when the default "coerce2" method fails to coerce an ordinary list,
+### we wrap the list in a SimpleList instance and try again. Note that this
+### should help in general because it brings 'from' a little bit closer to
+### 'class(to)'.
+setMethod("coerce2", "SimpleList",
+    function(from, to)
+    {
+        ans <- try(callNextMethod(), silent=TRUE)
+        if (!inherits(ans, "try-error"))
+            return(ans)
+        if (!is.list(from))
+            stop(wmsg(attr(ans, "condition")$message))
+        ## We use the SimpleList() constructor function to wrap 'from' in a
+        ## SimpleList instance instead of coercion to SimpleList (which is
+        ## too high level and tries to be too smart).
+        from <- SimpleList(from)
+        mcols(from) <- mcols(to)[rep.int(NA_integer_, length(from)), ,
+                                 drop=FALSE]
+        ans <- callNextMethod()
+        ## Even though coercion from SimpleList to 'class(to)' "worked", it
+        ## can return a broken object. This happens when an automatic coercion
+        ## method gets in the way. For example:
+        ##
+        ##   selectMethod("coerce", c("SimpleList", "BamFileList"))
+        ##
+        ## shows one of these methods (it's not coming from the Rsamtools or
+        ## S4Vectors package). The problem with these methods is that they
+        ## often do the wrong thing and don't even bother to validate the
+        ## object they return!
+        ## One known problem with the automatic coercion method from SimpleList
+        ## to one of its subclass is that it sets the elementType slot to "ANY"
+        ## which is generally wrong. So we fix this.
+        ans@elementType <- to@elementType
+        validObject(ans)
+        ans
+    }
+)
 
 .as.list.SimpleList <- function(x, use.names=TRUE)
 {
@@ -205,7 +218,7 @@ coerceToSimpleList <- function(from, element.type, ...) {
     if (is(from, "List"))
       element.type <- from@elementType
     else if (is.list(from))
-      element.type <- listElementType(from)
+      element.type <- lowestListElementClass(from)
     else element.type <- class(from)
   }
   SimpleListClass <- listClassName("Simple", element.type)
@@ -219,6 +232,7 @@ coerceToSimpleList <- function(from, element.type, ...) {
   }
 }
 
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### unique()
 ###
@@ -228,3 +242,4 @@ coerceToSimpleList <- function(from, element.type, ...) {
     as(lapply(x, unique, incomparables=incomparables, ...), class(x))
 }
 setMethod("unique", "SimpleList", .unique.SimpleList)
+
