@@ -53,7 +53,7 @@ setMethod("parallelVectorNames", "ANY",
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Getters.
+### Getters
 ###
 
 setMethod("length", "Vector",
@@ -111,8 +111,9 @@ setMethod("anyNA", "Vector", function(x, recursive=FALSE) any(is.na(x)))
 
 setMethod("is.na", "Vector", function(x) rep(FALSE, length(x)))
 
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Validity.
+### Validity
 ###
 
 .valid.Vector.length <- function(x)
@@ -141,19 +142,19 @@ setMethod("is.na", "Vector", function(x) rep(FALSE, length(x)))
                  "must be \"elementMetadata\"")
         return(paste(msg, collapse=""))
     }
-    for (slotname in x_pslotnames) {
+    msg <- NULL
+    for (slotname in head(x_pslotnames, -1L)) {
         tmp <- slot(x, slotname)
         if (!(is.null(tmp) || NROW(tmp) == x_len)) {
-            if (slotname == "elementMetadata") {
-                what <- "mcols(x)"
-            } else {
-                what <- paste0("x@", slotname)
-            }
-            msg <- c("'", what, "' is not parallel to 'x'")
-            return(paste(msg, collapse=""))
+            what <- paste0("x@", slotname)
+            msg <- c(msg, paste0("'", what, "' is not parallel to 'x'"))
         }
     }
-    NULL
+    tmp <- mcols(x)
+    if (!(is.null(tmp) || nrow(tmp) == x_len)) {
+        msg <- c(msg, "'mcols(x)' is not parallel to 'x'")
+    }
+    msg
 }
 
 .valid.Vector.names <- function(x)
@@ -203,7 +204,22 @@ setValidity2("Vector", .valid.Vector)
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Coercion.
+### updateObject()
+###
+### The default method (defined in BiocGenerics) does complicated, costly,
+### and dangerous things, and sometimes it actually breaks valid objects
+### (e.g. it breaks valid OverlapEncodings objects). So we overwrite it with
+### a method for Vector objects that does nothing! That way it's simple,
+### cheap, and safe ;-). And that's really all it needs to do at the moment.
+###
+
+setMethod("updateObject", "Vector",
+    function(object, ..., verbose=FALSE) object
+)
+
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Coercion
 ###
 
 setMethod("as.logical", "Vector",
@@ -294,8 +310,9 @@ setMethod("as.env", "Vector", function(x, enclos, tform = identity) {
 as.list.Vector <- function(x, ...) as.list(x, ...)
 setMethod("as.list", "Vector", function(x, ...) as.list(as(x, "List"), ...))
 
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Setters.
+### Setters
 ###
 
 setGeneric("elementMetadata<-",
@@ -305,8 +322,7 @@ setGeneric("elementMetadata<-",
 ### SummarizedExperiment, GenomicAlignments, and maybe more...
 ### 3x faster than new("DataFrame", nrows=nrow).
 ### 500x faster than DataFrame(matrix(nrow=nrow, ncol=0L)).
-make_zero_col_DataFrame <- function(nrow)
-    new2("DataFrame", nrows=nrow, check=FALSE)
+make_zero_col_DataFrame <- function(nrow) new_DataFrame(nrows=nrow)
 
 .normalize_mcols_replacement_value <- function(value, x)
 {
@@ -316,13 +332,14 @@ make_zero_col_DataFrame <- function(nrow)
     if (!("elementMetadata" %in% names(x_slots)))
         stop(wmsg("trying to set metadata columns on an object that does ",
                   "not support them (i.e. with no 'elementMetadata' slot)"))
-    mcols_class <- x_slots[["elementMetadata"]]
+    target_class <- x_slots[["elementMetadata"]]
     if (is.null(value)) {
-        if (is(NULL, mcols_class))
+        if (is(NULL, target_class))
             return(NULL)
         value <- make_zero_col_DataFrame(length(x))
     }
-    value <- as(value, mcols_class, strict=TRUE)
+    if (!is(value, target_class))
+        value <- as(value, target_class)
     ## From here 'value' is guaranteed to be a DataTable object.
     if (!is.null(rownames(value)))
         rownames(value) <- NULL
@@ -369,23 +386,51 @@ setGeneric("rename", function(x, ...) standardGeneric("rename"))
 setMethod("rename", "vector", .renameVector)
 setMethod("rename", "Vector", .renameVector)
 
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Subsetting.
+### Subsetting
 ###
 ### The "[" and "[<-" methods for Vector objects are just delegating to
 ### extractROWS() and replaceROWS() for performing the real work. Most of
-### the times, the author of a Vector subclass only needs to implement an
-### "extractROWS" and "replaceROWS" method for his/her objects.
+### the times, the author of a Vector subclass only needs to implement
+### an "extractROWS" and "replaceROWS" method for his/her objects.
 ###
 
-setMethod("[", "Vector", subset_along_ROWS)
+### The "[" method for Vector objects supports the 'x[i, j]' form to
+### allow the user to conveniently subset the metadata columns thru 'j'.
+### Note that a Vector subclass with a true 2-D semantic (e.g.
+### SummarizedExperiment) needs to overwrite this. This means that code
+### intended to operate on an arbitrary Vector derivative 'x' should not
+### use this feature as there is no guarantee that 'x' supports it. For
+### this reason this feature should preferrably be used interactively only.
+setMethod("[", "Vector",
+    function(x, i, j, ..., drop=TRUE)
+    {
+        ans <- subset_along_ROWS(x, i, , ..., drop=drop)
+        if (!missing(j))
+            mcols(ans) <- mcols(ans)[ , j, drop=FALSE]
+        ans
+    }
+)
 
-### We provide a default "extractROWS" method for Vector objects that only
-### subsets the individual parallel slots. That should be enough for most
-### Vector derivatives that have parallelSlotNames() properly set.
+### We provide a default "extractROWS" method for Vector objects that subsets
+### all the parallel slots.
+### Note that this method will work out-of-the-box and do the right thing
+### on most Vector subclasses as long as parallelSlotNames() reports the
+### names of all the parallel slots on objects of the subclass (some Vector
+### subclasses might require a "parallelSlotNames" method for this to happen).
+### For those Vector subclasses on which extractROWS() does not work
+### out-of-the-box nor do the right thing, it is strongly advised to override
+### the method for Vector objects rather than trying to override the "["
+### method for Vector objects with a specialized method. The specialized
+### "extractROWS" method will typically delegate to the method below via the
+### use of callNextMethod(). See "extractROWS" method for Hits for an example.
 setMethod("extractROWS", "Vector",
     function(x, i)
     {
+        ## Fix old objects on-the-fly (e.g. old GRanges or GAlignments
+        ## instances).
+        x <- updateObject(x, check=FALSE)
         i <- normalizeSingleBracketSubscript(i, x, as.NSBS=TRUE)
         x_pslotnames <- parallelSlotNames(x)
         ans_pslots <- lapply(setNames(x_pslotnames, x_pslotnames),
@@ -425,7 +470,7 @@ setReplaceMethod("[", "Vector",
     }
 )
 
-### Work on any Vector object for which c() and extractROWS() work.
+### Work on any Vector object on which c() and extractROWS() work.
 ### Assume 'value' is compatible with 'x'.
 setMethod("replaceROWS", "Vector",
     function(x, i, value)
@@ -434,10 +479,11 @@ setMethod("replaceROWS", "Vector",
 
         ## --<1>-- Concatenate 'x' and 'value' with c() -----
 
-        ## We assume that c() works on objects of class 'class(x)' and that it
-        ## does the right thing i.e. that it returns an object of the same
-        ## class as 'x' and of length 'length(x) + length(value)'.
-        ans <- c(x, value)
+        ## We assume that concatenateObjects() works on objects of
+        ## class 'class(x)' and does the right thing i.e. that it
+        ## returns an object of the same class as 'x' and of length
+        ## 'length(x) + length(value)'. We skip validation.
+        ans <- concatenateObjects(x, list(value), check=FALSE)
 
         ## --<2>-- Subset 'c(x, value)' with extractROWS() -----
 
@@ -448,6 +494,8 @@ setMethod("replaceROWS", "Vector",
         ## trying to normalize it.
         idx <- NativeNSBS(idx, length(ans), TRUE, FALSE)
         ## We assume that extractROWS() works on an object of class 'class(x)'.
+        ## For some objects (e.g. Hits), extractROWS() will take care of
+        ## validating the returned object.
         ans <- extractROWS(ans, idx)
 
         ## --<3>-- Restore the original decoration -----
@@ -466,9 +514,22 @@ setMethod("replaceROWS", "Vector",
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Convenience wrappers for common subsetting operations.
+### Convenience wrappers for common subsetting operations
 ###
 
+### S3/S4 combo for subset.Vector
+subset.Vector <- function(x, ...) subset(x, ...)
+subset_Vector <- function(x, subset, select, drop=FALSE, ...)
+{
+    i <- evalqForSubset(subset, x, ...)
+    if (!is.null(mcols(x))) {
+        j <- evalqForSelect(select, mcols(x), ...)
+        mcols(x) <- mcols(x)[ , j, drop=FALSE]
+    }
+    x[i, drop=drop]
+}
+setMethod("subset", "Vector", subset_Vector)
+
 ### S3/S4 combo for window.Vector
 window.Vector <- function(x, ...) window(x, ...)
 Vector_window <- function(x, start=NA, end=NA, width=NA)
@@ -493,7 +554,10 @@ revROWS <- function(x) extractROWS(x, rev(seq_len(NROW(x))))
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Combining.
+### Concatenation
+###
+### Note that supporting "extractROWS" and "c" makes "replaceROWS" (and thus
+### "[<-") work out-of-the-box!
 ###
 
 ### Somewhat painful that we do not always have a DataFrame in elementMetadata
@@ -511,9 +575,9 @@ rbind_mcols <- function(x, ...)
     mcols_list <- lapply(args, mcols)
     if (length(mcols_list) == 1L)
         return(mcols_list[[1L]])
-    mcols_is_null <- sapply(mcols_list, is.null)
+    mcols_is_null <- sapply_isNULL(mcols_list)
     if (all(mcols_is_null))
-        return(NULL)    
+        return(NULL)
     mcols_list[mcols_is_null] <- lapply(
         args[mcols_is_null],
         function(arg) make_zero_col_DataFrame(length(arg))
@@ -528,13 +592,98 @@ rbind_mcols <- function(x, ...)
     do.call(rbind, lapply(mcols_list, fillCols))
 }
 
-rbindRowOfNAsToMetadatacols <- function(x) {
-  x_mcols <- mcols(x)
-  if (!is.null(x_mcols))
-    mcols(x)[nrow(x_mcols)+1L,] <- NA
-  x
+### We provide a default "concatenateObjects" method for Vector objects that
+### concatenates all the parallel slots. The method behaves like
+### an endomorphism with respect to its first argument 'x'.
+### Note that this method will work out-of-the-box and do the right thing
+### on most Vector subclasses as long as parallelSlotNames() reports the
+### names of all the parallel slots on objects of the subclass (some Vector
+### subclasses might require a "parallelSlotNames" method for this to happen).
+### For those Vector subclasses on which concatenateObjects() does not work
+### out-of-the-box nor do the right thing, it is strongly advised to override
+### the method for Vector objects rather than trying to override the "c"
+### method for Vector objects with a specialized method. The specialized
+### "concatenateObjects" method will typically delegate to the method below
+### via the use of callNextMethod(). See "concatenateObjects" methods for
+### Hits and Rle objects for some examples.
+### No Vector subclass should need to override the "c" method for
+### Vector objects.
+.concatenate_Vector_objects <-
+    function(x, objects=list(), use.names=TRUE, ignore.mcols=FALSE, check=TRUE)
+{
+    if (!isTRUEorFALSE(use.names))
+        stop("'use.names' must be TRUE or FALSE")
+    if (!isTRUEorFALSE(ignore.mcols))
+        stop("'ignore.mcols' must be TRUE or FALSE")
+    if (!isTRUEorFALSE(check))
+        stop("'check' must be TRUE or FALSE")
+
+    objects <- prepare_objects_to_concatenate(x, objects)
+    all_objects <- c(list(x), objects)
+
+    ## Concatenate all the parallel slots except "NAMES" and "elementMetadata".
+    x_pslotnames <- parallelSlotNames(x)
+    pslotnames <- setdiff(x_pslotnames, c("NAMES", "elementMetadata"))
+    ans_pslots <- lapply(setNames(pslotnames, pslotnames),
+        function(slotname) {
+            x_slot <- slot(x, slotname)
+            if (is.null(x_slot))
+                return(NULL)
+            slot_list <- lapply(objects, slot, slotname)
+            concatenateObjects(x_slot, slot_list)
+        }
+    )
+
+    if ("NAMES" %in% x_pslotnames) {
+        ans_NAMES <- NULL
+        if (use.names) {
+            names_list <- lapply(all_objects, slot, "NAMES")
+            object_has_no_names <- sapply_isNULL(names_list)
+            if (!all(object_has_no_names)) {
+                ## Concatenate the "NAMES" slots.
+                names_list[object_has_no_names] <-
+                    lapply(all_objects[object_has_no_names],
+                           function(object) character(length(object)))
+                ans_NAMES <- unlist(names_list, use.names=FALSE)
+            }
+        }
+        ans_pslots <- c(ans_pslots, list(NAMES=ans_NAMES))
+    }
+
+    if (!ignore.mcols) {
+        ## Concatenate the "elementMetadata" slots.
+        ans_mcols <- do.call(rbind_mcols, all_objects)
+        ans_pslots <- c(ans_pslots, list(elementMetadata=ans_mcols))
+    }
+
+    ans <- do.call(BiocGenerics:::replaceSlots,
+                   c(list(x), ans_pslots, list(check=FALSE)))
+
+    if (ignore.mcols)
+        mcols(ans) <- NULL
+
+    if (check)
+        validObject(ans)
+    ans
 }
 
+setMethod("concatenateObjects", "Vector", .concatenate_Vector_objects)
+
+### Thin wrapper around concatenateObjects(). Behave like an endomorphism
+### i.e. return an object of the same class as 'x'. In particular 'c(x)'
+### should return 'x'.
+### No Vector subclass should need to override this method. See the
+### "concatenateObjects" method for Vector objects above for more information.
+setMethod("c", "Vector",
+    function(x, ..., ignore.mcols=FALSE, recursive=FALSE)
+    {
+        if (!identical(recursive, FALSE))
+            stop(wmsg("\"c\" method for Vector objects ",
+                      "does not support the 'recursive' argument"))
+        concatenateObjects(x, list(...), ignore.mcols=ignore.mcols)
+    }
+)
+
 ### FIXME: This method doesn't work properly on DataTable objects if 'after'
 ### is >= 1 and < length(x).
 setMethod("append", c("Vector", "Vector"),
@@ -554,7 +703,7 @@ setMethod("append", c("Vector", "Vector"),
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Evaluating.
+### Evaluating
 ###
 
 setMethod("eval", c("expression", "Vector"),
@@ -587,8 +736,9 @@ transform.Vector <- transformColumns
 
 setMethod("transform", "Vector", transform.Vector)
 
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Utilities.
+### Utilities
 ###
 
 setGeneric("expand.grid", signature="...")
@@ -624,3 +774,4 @@ setMethod("by", "Vector",
           })
 
 diff.Vector <- function(x, ...) diff(x, ...)
+
