@@ -4,6 +4,19 @@
 ###
 
 
+.match_name <- function(i, x_names, exact=TRUE)
+{
+    if (exact) {
+        match(i, x_names, incomparables=c(NA_character_, ""))
+    } else {
+        ## When 'i' has length 1, it doesn't matter whether we use
+        ## 'duplicates.ok=FALSE' (the default) or 'duplicates.ok=TRUE' but
+        ## the latter seems to be just a little bit faster.
+        pmatch(i, x_names, duplicates.ok=TRUE)
+    }
+}
+
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Formal representation of a Normalized Single Bracket Subscript, i.e. a
 ### subscript that holds positive integer values that can be used for single
@@ -12,7 +25,7 @@
 ### NSBS and its subclasses are for internal use only.
 ###
 
-setClass("NSBS", 
+setClass("NSBS",
     representation(
         "VIRTUAL",
         ## 'subscript' is an object that holds integer values >= 1 and
@@ -80,10 +93,6 @@ setMethod("NSBS", "NSBS",
     }
 )
 
-### NSBS concrete subclasses NativeNSBS, RangeNSBS, and RleNSBS override this
-### default method.
-setMethod("as.integer", "NSBS", function(x) as.integer(x@subscript))
-
 ### The 3 default methods below work out-of-the-box on NSBS objects for which
 ### as.integer() works. However, concrete subclasses RangeNSBS, RleNSBS, and
 ### RangesNSBS override some of them with more efficient versions that avoid
@@ -116,7 +125,7 @@ setClass("NativeNSBS",  # not exported
     )
 )
 
-### Construction methods.
+### Construction.
 ### Supplied arguments are trusted so we don't check them!
 
 NativeNSBS <- function(subscript, upper_bound, upper_bound_is_strict, has_NAs)
@@ -126,15 +135,6 @@ NativeNSBS <- function(subscript, upper_bound, upper_bound_is_strict, has_NAs)
                        has_NAs=has_NAs,
                        check=FALSE)
 
-setMethod("NSBS", "missing",
-    function(i, x, exact=TRUE, strict.upper.bound=TRUE, allow.NAs=FALSE)
-    {
-        x_NROW <- NROW(x)
-        i <- seq_len(x_NROW)
-        NativeNSBS(i, x_NROW, strict.upper.bound, FALSE)
-    }
-)
-
 setMethod("NSBS", "NULL",
     function(i, x, exact=TRUE, strict.upper.bound=TRUE, allow.NAs=FALSE)
     {
@@ -144,12 +144,16 @@ setMethod("NSBS", "NULL",
     }
 )
 
-.NSBS.numeric <- function(i, x, exact=TRUE, strict.upper.bound=TRUE,
-                                allow.NAs=FALSE)
+.NSBS.numeric <- function(i, x, exact=TRUE,
+                          strict.upper.bound=TRUE, allow.NAs=FALSE)
 {
     x_NROW <- NROW(x)
-    if (!is.integer(i))
-        i <- as.integer(i)
+    if (is.integer(i)) {
+        if (!is.null(names(i)))
+            names(i) <- NULL
+    } else {
+        i <- as.integer(i)  # this also drops the names
+    }
     has_NAs <- anyNA(i)
     if (!allow.NAs && has_NAs)
         .subscript_error("subscript contains NAs")
@@ -178,6 +182,8 @@ setMethod("NSBS", "logical",
         x_NROW <- NROW(x)
         if (anyNA(i))
             .subscript_error("logical subscript contains NAs")
+        if (!is.null(names(i)))
+            names(i) <- NULL
         li <- length(i)
         if (strict.upper.bound && li > x_NROW) {
             if (any(i[(x_NROW+1L):li]))
@@ -194,8 +200,8 @@ setMethod("NSBS", "logical",
     }
 )
 
-.NSBS.characterORfactor <- function(i, x, exact=TRUE, strict.upper.bound=TRUE,
-                                          allow.NAs=FALSE)
+.NSBS.character_OR_factor <- function(i, x, exact=TRUE,
+                                      strict.upper.bound=TRUE, allow.NAs=FALSE)
 {
     x_NROW <- NROW(x)
     x_ROWNAMES <- ROWNAMES(x)
@@ -207,11 +213,7 @@ setMethod("NSBS", "logical",
         i <- x_NROW + seq_along(i)
         return(NativeNSBS(i, x_NROW, FALSE, FALSE))
     }
-    if (exact) {
-        i <- match(i, x_ROWNAMES, incomparables=c(NA_character_, ""))
-    } else {
-        i <- pmatch(i, x_ROWNAMES, duplicates.ok=TRUE)
-    }
+    i <- .match_name(i, x_ROWNAMES, exact=exact)
     if (!strict.upper.bound) {
         na_idx <- which(is.na(i))
         i[na_idx] <- x_NROW + seq_along(na_idx)
@@ -223,9 +225,9 @@ setMethod("NSBS", "logical",
     NativeNSBS(i, x_NROW, strict.upper.bound, has_NAs)
 }
 
-setMethod("NSBS", "character", .NSBS.characterORfactor)
+setMethod("NSBS", "character", .NSBS.character_OR_factor)
 
-setMethod("NSBS", "factor", .NSBS.characterORfactor)
+setMethod("NSBS", "factor", .NSBS.character_OR_factor)
 
 setMethod("NSBS", "array",
     function(i, x, exact=TRUE, strict.upper.bound=TRUE, allow.NAs=FALSE)
@@ -238,6 +240,7 @@ setMethod("NSBS", "array",
 
 ### Other methods.
 
+### We override the "as.integer" default method for NSBS objects.
 setMethod("as.integer", "NativeNSBS", function(x) x@subscript)
 
 
@@ -255,7 +258,7 @@ setClass("RangeNSBS",  # not exported
     )
 )
 
-### Constructor.
+### Construction.
 
 .normarg_range_start <- function(start, argname="start")
 {
@@ -303,6 +306,20 @@ RangeNSBS <- function(x, start=NA, end=NA, width=NA)
                       check=FALSE)
 }
 
+setMethod("NSBS", "missing",
+    function(i, x, exact=TRUE, strict.upper.bound=TRUE, allow.NAs=FALSE)
+    {
+        RangeNSBS(x, start=1L, end=NROW(x))
+    }
+)
+
+
+### Other methods.
+
+### We override the "as.integer", "length", "anyDuplicated", and
+### "isStrictlySorted" default methods for NSBS objects with more
+### efficient ones.
+
 setMethod("as.integer", "RangeNSBS",
     function(x)
     {
@@ -349,9 +366,11 @@ setMethod("show", "RangeNSBS",
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### normalizeSingleBracketSubscript()
 ###
+### Must return an unnamed integer vector when 'as.NSBS' is FALSE.
+###
 
-normalizeSingleBracketSubscript <- function(i, x,
-                                            exact=TRUE, allow.append=FALSE,
+normalizeSingleBracketSubscript <- function(i, x, exact=TRUE,
+                                            allow.append=FALSE,
                                             allow.NAs=FALSE,
                                             as.NSBS=FALSE)
 {
@@ -390,12 +409,12 @@ setMethod("normalizeSingleBracketReplacementValue", "ANY",
     {
         if (is(value, class(x)))
             return(value)
-        lv <- length(value)
+        value_len <- length(value)
         value <- try(as(value, class(x)), silent=TRUE)
         if (inherits(value, "try-error"))
             stop("'value' must be a ", class(x), " object (or coercible ",
                  "to a ", class(x), " object)")
-        if (length(value) != lv)
+        if (length(value) != value_len)
             stop("coercing replacement value to ", class(x), "\n",
                  "  changed its length!\n",
                  "  Please do the explicit coercion ",
@@ -431,7 +450,7 @@ setGeneric("replaceROWS", signature="x",
     function(x, i, value) standardGeneric("replaceROWS")
 )
 
-.extractROWSWithBracket <- function(x, i)
+default_extractROWS <- function(x, i)
 {
   if (is.null(x) || missing(i))
     return(x)
@@ -444,7 +463,7 @@ setGeneric("replaceROWS", signature="x",
   do.call(`[`, args)
 }
 
-.replaceROWSWithBracket <- function(x, i, value)
+default_replaceROWS <- function(x, i, value)
 {
   if (is.null(x))
     return(x)
@@ -456,7 +475,7 @@ setGeneric("replaceROWS", signature="x",
   do.call(`[<-`, args)
 }
 
-setMethod("extractROWS", c("ANY", "ANY"), .extractROWSWithBracket)
+setMethod("extractROWS", c("ANY", "ANY"), default_extractROWS)
 
 ### NOT exported but used in IRanges package (by "extractROWS" method with
 ### signature vector_OR_factor,RangesNSBS).
@@ -475,8 +494,8 @@ setMethod("extractROWS", c("vector_OR_factor", "RangeNSBS"),
     }
 )
 
-setMethod("extractROWS", c("array", "RangeNSBS"), .extractROWSWithBracket)
-setMethod("extractROWS", c("array", "RangesNSBS"), .extractROWSWithBracket)
+setMethod("extractROWS", c("array", "RangeNSBS"), default_extractROWS)
+setMethod("extractROWS", c("data.frame", "RangeNSBS"), default_extractROWS)
 
 ### NOT exported but will be used in IRanges package (by "extractROWS" method
 ### with signature LLint,RangesNSBS).
@@ -528,71 +547,96 @@ subset_along_ROWS <- function(x, i, j, ..., drop=TRUE)
 
 setMethod("[", "LLint", subset_along_ROWS)
 
-setMethod("replaceROWS", "ANY", .replaceROWSWithBracket)
+setMethod("replaceROWS", "ANY", default_replaceROWS)
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### normalizeDoubleBracketSubscript()
 ###
-### Supported types for 'i': single NA, or numeric or character vector of
-### length 1, or numeric- or character-Rle of length 1.
-### Always returns a single integer. When called with 'error.if.nomatch=FALSE',
-### returns an NA_integer_ if no match is found. Otherwise (the default),
-### raises an error if no match is found so the returned integer is guaranteed
-### to be a non-NA positive integer referring to a valid position in 'x'.
+### The supplied subscript 'i' must represent (1) a single non-NA number,
+### or (2) a single non-NA string, or (3) a single NA (only if 'allow.NA'
+### is TRUE). It must be represented as an ordinary atomic vector or Rle
+### object of length 1. More precisely:
+###  (1) A single non-NA number must be represented as an integer or numeric
+###      vector of length 1, or as an integer- or numeric-Rle object of
+###      length 1. It must be >= 1 and <= length(x), except if 'allow.append'
+###      is TRUE, in which case it must be >= 1 and <= length(x) + 1.
+###      If these conditions are satisfied, the subscript is returned as a
+###      single integer. Otherwise an error is raised.
+###  (2) A single non-NA string must be represented as a character vector or
+###      factor of length 1, or as a character- or factor-Rle object of
+###      length 1. It must match a name on 'x', except if 'allow.nomatch' is
+###      TRUE, in which case it doesn't have to match a name on 'x'.
+###      If these conditions are satisfied, the position of the match or NA
+###      is returned. Otherwise an error is raised.
+###  (3) A single NA must be represented as an atomic vector (of any type)
+###      or Rle object of length 1. It is returned as a single logical NA.
+### Return a single integer that is >= 1 and <= length(x).
 ###
 
 normalizeDoubleBracketSubscript <- function(i, x, exact=TRUE,
-                                            error.if.nomatch=TRUE)
+                                            allow.append=FALSE,
+                                            allow.NA=FALSE,
+                                            allow.nomatch=FALSE)
 {
-    if (!isTRUEorFALSE(exact))
-        stop("'exact' must be TRUE or FALSE")
-    if (!isTRUEorFALSE(error.if.nomatch))
-        stop("'error.if.nomatch' must be TRUE or FALSE")
     if (missing(i))
         stop("subscript is missing")
+    if (!isTRUEorFALSE(exact))
+        stop("'exact' must be TRUE or FALSE")
+    if (!isTRUEorFALSE(allow.append))
+        stop("'allow.append' must be TRUE or FALSE")
+    if (!isTRUEorFALSE(allow.NA))
+        stop("'allow.NA' must be TRUE or FALSE")
+    if (!isTRUEorFALSE(allow.nomatch))
+        stop("'allow.nomatch' must be TRUE or FALSE")
     subscript_type <- class(i)
     if (is(i, "Rle")) {
         i <- decodeRle(i)
         subscript_type <- paste0(class(i), "-", subscript_type)
     }
+    if (is.factor(i))
+        i <- as.character(i)
     if (is.vector(i) && length(i) == 1L && is.na(i)) {
-        if (error.if.nomatch)
-            stop("subsetting by NA returns no match")
-        return(NA_integer_)
+        if (!allow.NA)
+            stop("NA is not a valid [[ subscript")
+        return(NA)
     }
-    if (!is.numeric(i) && !is.character(i))
+    if (!(is.numeric(i) || is.character(i)))
         stop("invalid [[ subscript type: ", subscript_type)
     if (length(i) < 1L)
         stop("attempt to extract less than one element")
     if (length(i) > 1L)
         stop("attempt to extract more than one element")
+    x_len <- length(x)
     if (is.numeric(i)) {
         if (!is.integer(i))
             i <- as.integer(i)
-        if (i < 1L || length(x) < i)
-            stop("subscript is out of bounds")
+        if (i < 1L)
+            stop("[[ subscript must be >= 1")
+        if (allow.append) {
+            if (i > x_len + 1L)
+                stop("[[ subscript must be <= length(x) + 1")
+        } else {
+            if (i > x_len)
+                stop("subscript is out of bounds")
+        }
         return(i)
     }
-    ## 'i' is a character string
+    ## 'i' is a single non-NA string.
     x_names <- names(x)
     if (is.null(x_names)) {
-        if (error.if.nomatch)
+        if (!allow.nomatch)
             stop("attempt to extract by name when elements have no names")
-        return(NA_integer_)
+        return(NA)
     }
     #if (i == "")
     #    stop("invalid subscript \"\"")
-    if (exact) {
-        ans <- match(i, x_names, incomparables=c(NA_character_, ""))
-    } else {
-        ## Because 'i' has length 1, it doesn't matter whether we use
-        ## 'duplicates.ok=FALSE' (the default) or 'duplicates.ok=TRUE' but
-        ## the latter seems to be just a little bit faster.
-        ans <- pmatch(i, x_names, duplicates.ok=TRUE)
+    ans <- .match_name(i, x_names, exact=exact)
+    if (is.na(ans)) {
+        if (!allow.nomatch)
+            stop("subscript \"", i, "\" matches no name")
+        return(NA)
     }
-    if (is.na(ans) && error.if.nomatch)
-        stop("subscript \"", i, "\" matches no name")
     ans
 }
 
@@ -610,15 +654,101 @@ setGeneric("setListElement", signature="x",
     function(x, i, value) standardGeneric("setListElement")
 )
 
+### Note that although is(x, "list") is FALSE on a data.frame (a non-sense
+### that some people will find a way to justify), dispatch will call this
+### method if 'x' is a data.frame.
 setMethod("getListElement", "list",
     function(x, i, exact=TRUE)
     {
-        i <- normalizeDoubleBracketSubscript(i, x, exact=exact,
-                                             error.if.nomatch=FALSE)
-        x[[i]]
+        i2 <- normalizeDoubleBracketSubscript(i, x, exact=exact,
+                                              allow.NA=TRUE,
+                                              allow.nomatch=TRUE)
+        if (is.na(i2))
+            return(NULL)
+        x[[i2]]
     }
 )
 
+### Based on `[`. This should automatically take care of removing the
+### corresponding row in 'mcols(x)' if 'x' is a Vector derivative.
+.remove_list_element <- function(x, i)
+{
+    stopifnot(isSingleNumberOrNA(i))
+    if (is.na(i) || i < 1L || i > length(x))
+        return(x)  # no-op
+    ## `[<-.data.frame` does some terrible mangling of the colnames
+    ## if they contain duplicates so we can't use it here.
+    if (is.data.frame(x)) {
+        x[[i]] <- NULL
+        return(x)
+    }
+    x[-i]
+}
+
+.wrap_in_length_one_list_like_object <- function(value, name, x)
+{
+    stopifnot(is(x, "list_OR_List"))
+    stopifnot(is.null(name) || isSingleStringOrNA(name))
+    if (is(x, "List")) {
+        tmp <- try(as(value, elementType(x), strict=FALSE), silent=TRUE)
+        if (!inherits(tmp, "try-error"))
+            value <- tmp
+    }
+    value <- setNames(list(value), name)
+    value <- try(coerce2(value, x), silent=TRUE)
+    if (inherits(value, "try-error"))
+        stop(wmsg("failed to coerce 'list(value)' to a ", class(x),
+                  " object of length 1"))
+    value
+}
+
+### Based on 'c()'. This should automatically take care of adjusting the
+### metadata columns (by rbind'ing a row of NAs to 'mcols(x)') if 'x' is
+### a Vector derivative.
+.append_list_element <- function(x, value, name=NULL)
+{
+    if (is.null(name) && !is.null(names(x)))
+        name <- ""
+    value <- .wrap_in_length_one_list_like_object(value, name, x)
+    coerce2(c(x, value), x)
+}
+
+### Based on `[<-`.
+.replace_list_element <- function(x, i, value)
+{
+    value <- .wrap_in_length_one_list_like_object(value, names(x)[[i]], x)
+    ## `[<-` propagates the metadata columns from 'value' to 'x' but here
+    ## we don't want that.
+    if (is(x, "Vector"))
+        x_mcols <- mcols(x)
+    x[i] <- value
+    if (is(x, "Vector"))
+        mcols(x) <- x_mcols
+    x
+}
+
+### Work on any list-like object for which `[<-`, c(), and `[` work.
+### Also, if 'value' is not NULL, 'list(value)' must be coercible to a
+### length-one list-like object of the same class as 'x'.
+setListElement_default <- function(x, i, value)
+{
+    i2 <- normalizeDoubleBracketSubscript(i, x,
+                                          allow.append=TRUE,
+                                          allow.nomatch=TRUE)
+    if (is.null(value))
+        return(.remove_list_element(x, i2))
+    if (is.na(i2) || i2 > length(x)) {
+        name <- if (is.na(i2)) as.character(i) else NULL
+        return(.append_list_element(x, value, name))
+    }
+    .replace_list_element(x, i2, value)
+}
+
+### Note that although is(x, "list") is FALSE on a data.frame (a non-sense
+### that some people will find a way to justify), dispatch will call this
+### method if 'x' is a data.frame.
+setMethod("setListElement", "list", setListElement_default)
+
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### window(), head(), tail(), rep.int()
