@@ -502,6 +502,13 @@ int _is_LLint(SEXP x);
 
 SEXP make_RAW_from_NA_LLINT();
 
+int sscan_llint(
+	const char *s,
+	int maxparse,
+	long long int *val,
+	int parse_dec
+);
+
 R_xlen_t _get_LLint_length(SEXP x);
 
 long long int *_get_LLint_dataptr(SEXP x);
@@ -610,6 +617,11 @@ SEXP logical2_sum(
 
 /* integer_utils.c */
 
+SEXP to_list_of_ints(
+	SEXP x,
+	SEXP sep
+);
+
 SEXP Integer_any_missing_or_outside(SEXP x, SEXP lower, SEXP upper);
 
 SEXP Integer_diff_with_0(SEXP x);
@@ -796,8 +808,6 @@ SEXP unstrsplit_list(SEXP x, SEXP sep);
 
 SEXP safe_strexplode(SEXP s);
 
-SEXP strsplit_as_list_of_ints(SEXP x, SEXP sep);
-
 SEXP svn_time();
 
 
@@ -867,6 +877,7 @@ SEXP map_positions(
 /* Hits_class.c */
 
 SEXP _new_Hits(
+	const char *Class,
 	int *from,
 	const int *to,
 	int nhit,
